////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "bxdfblinn.h"


//
// approximation of the blinn specular model, which does not require the computation of
// the micro-distribution facets and is thus, cheaper to calculate.
//

CBxDFBlinn::CBxDFBlinn(void) : CBxDFBase( static_cast<BxDFType>( BSDF_DIFFUSE | BSDF_SPECULAR ) )
{
}

CBxDFBlinn::~CBxDFBlinn(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFBlinn::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Shade ); UNUSED( Power ); UNUSED( RenderMat ); UNUSED( LightColor );

  double          Rho = 0.0;
  Vector          L = In.dir();        // light vector
  Vector          OutDir = -Out.dir();
  const double    cEta = 4.0;             // This parameter controls the "spread" of the specular highlight

  // calculate diffuse shading
  if( RenderMat.specular() > 0 ) {
    double CosTIn = L.dot( Normal );

    if( CosTIn <= 0.0)
      return Color::Black;

    Vector H( OutDir + L);
    H.normalize();

    double nh = Normal.dot( H );
    if( nh <= 0.0 )
      return Color::Black;

    double CosTOut = OutDir.dot( Normal );
    double  vh = OutDir.dot( H );
    double ivh = (vh != 0.0) ? (1.0 / vh) : 0.0;

    double GFunc = std::min( 1.0, std::min( 2.0 * nh * CosTOut * ivh, 2.0 * nh * CosTIn * ivh ) );
    double g = sqrt( cEta * cEta + vh * vh - 1.0);
    double F = (sqrt( g - vh ) / sqrt( g + vh) ) * ( 1.0 + sqrt( vh * ( g + vh ) - 1.0) / sqrt( vh * (g - vh) + 1.0 ) );

    // TODO: Check the calculation of the specular power!
    double sp = (RenderMat.shine() < 100.) ? ( sqrt( 1.0 / (RenderMat.shine() * 0.3))) : ( 10.0 / (RenderMat.shine() * 0.3) );
    Rho = RenderMat.specular() / PI * F * GFunc * exp(-sqrt(acos(nh) ) / (2.0 * sp * sp) ) / CosTIn;
  }

  return LightColor * Rho;
}
// vim:ts=2
