////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "bxdfspecrefl.h"


CBxDFSpecReflection::CBxDFSpecReflection( CFresnel* pFresnel ) : CBxDFBase( static_cast<BxDFType>( BSDF_REFLECTION | BSDF_SPECULAR) ),
m_pFresnel( pFresnel)
{
  assert( m_pFresnel );
}

CBxDFSpecReflection::~CBxDFSpecReflection(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFSpecReflection::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( Out ); UNUSED( In ); UNUSED( Normal ); UNUSED( RenderMat ); UNUSED( LightColor );
	UNUSED( Shade ); UNUSED( Power );

	return LightColor;
}



////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFSpecReflection::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0 */ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( Shade ); UNUSED( Power ); UNUSED( LightColor );

  if( Type == CBxDFBase::BSDF_REFLECTION )
    return Sample_f_Reflection( Out, In, Pdf, Normal, RenderMat, pRandNr, LightColor );

  return Color::Black;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CBxDFSpecReflection::Sample_f_Reflection( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, const Color& LightColor )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( pRandNr );

  /////
  double EtaI = 1.0;    // should come from the out.ray
	double EtaT = RenderMat.ior();
	double ReflectionCoeff = 0;

	Pdf = 1.0f;

	// CT: we only render reflections if we are at the beauty pass (step = 1)
	double dot = Normal * Out.dir();
	if( dot < 0.0 ) {
		Vector reflect = (-2.0 * dot) * Normal + Out.dir();
		reflect.normalize();

		// Find intersection with light
		In = Out;
		In.init();
		In.depth = Out.depth + 1;
		In.push( RaySegment( Out.hit( -SMALL3D4 * 10.0 ), reflect, Out.prevZone() ) );

		ReflectionCoeff = m_pFresnel->Evaluate( fabs(dot), EtaI, EtaT );
		return LightColor * (ReflectionCoeff * RenderMat.specular() / fabs(dot));
	}

	return Color::Black;

}
// vim:ts=2
