////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MAT_FRESNEL_BLEND_H
#define MAT_FRESNEL_BLEND_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "bxdffresnel.h"
#include "bxdfmicrofacet.h"
#include "bxdfphong.h"
#include "blinnmicrodist.h"
#include "bxdfspecrefl.h"
#include "bxdfspectransmission.h"
#include "anisotropicmicrodist.h"
#include "bxdffresnelblend.h"
#include "bxdftranslucent.h"
#include "baseintegrator.h"
#include "noisetexture.h"

class CBxDFPhong;
class CBxDFFresnelBlend;
class CAnisotropicMicroDist;

class CMatFresnelBlend : public CPhysMatInterface
{
public:
  CMatFresnelBlend( CBaseIntegrator* pInt, double eX  = 4 , double eY  = 20 , bool bPerfectSpecularRefl = false, bool bNoiseTex  = false, double NoiseScaling  = 1 );
	CMatFresnelBlend( CBaseIntegrator* pInt, CMicrofacetDistribution* pMicroDist, bool bPerfectSpecularRefl = false, bool bNoiseTex = false, double NoiseScaling = 1 );

	virtual ~CMatFresnelBlend(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual Color f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0f ) override;

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual Color Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;

protected:
  CBxDFFresnelBlend*				m_pBxDFDiffuse;
	CMicrofacetDistribution*  m_pMicroDist;
	CNoiseTexture							m_NoiseTexture;
	bool											m_bUseNoise;
	double										m_NoiseScaling;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// substrate with image texture and displacement mapping
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CMatFresnelBlendTex : public CMatFresnelBlend
{
public:
	CMatFresnelBlendTex( CBaseIntegrator* pInt, CMicrofacetDistribution* pMicroDist, bool bPerfectSpecularRefl, const std::string& TexFileName, double TexDimU, double TexDimV, double NoiseScaling = 1 ) :
		CMatFresnelBlend( pInt, pMicroDist, bPerfectSpecularRefl, false, NoiseScaling )
	{
		m_TexFileName = TexFileName;
		m_TexDimU = TexDimU;
		m_TexDimV = TexDimV;

		//InitTextures();
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	virtual const Vector& GetShadingNormal( const Ray* pCurRay ) override
	{
		return GetBumpNormalFromTex( pCurRay );
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	virtual void ModifyShadingGeometry( Ray* pCurRay, GeometryEngine* pEngine ) override
	{
		return ModifyShadingGeometryByTex( pCurRay, pEngine );
	}

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// override color by blending the base color with the image texture
	Color GetHitColor( const Ray* pRay ) override
	{
		return MixBaseWithTexColor( pRay, 0.95 );
	}

};


#endif
// vim:ts=2
