////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once
#ifndef PERLIN_NOISE_H
#define PERLIN_NOISE_H

#include "math/vector.h"
#include "math/random.h"
#include <array>

constexpr int cNrOfPerlinPoints = 256;

class CPerlinNoise
{
public:
	explicit CPerlinNoise();

public:
	double Noise( const Vector& p ) const;
	double Turbulence( const Vector& p, const int NoiseDepth = 7 ) const;

protected:

	static void Shuffle( Random& RNG, int* p, int n );
	double			PerlinInterpolation( const Vector c[2][2][2], const double u, const double v, const double w ) const;

protected:
	std::array<Vector, cNrOfPerlinPoints> m_RandomVectors;
	std::array<int, cNrOfPerlinPoints>		m_Permutation[3];
	Random																m_RNG;
};

#endif