/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Adapted from halotrace written by C. Theis, 2007
 *
 * Author:	Christian.Theis@cern.ch
 *
 */

#include "filter.h"

CFilter::CFilter( double XWidth, double YWidth, int NrOfSamples ) : m_NrOf1DSamples( NrOfSamples ), m_Width( XWidth, YWidth ), m_InvWidthX( 1.0 / XWidth ), m_InvWidthY( 1.0 / YWidth )
{
  if( m_NrOf1DSamples != 0) {
    m_TotalSamples = m_NrOf1DSamples * m_NrOf1DSamples;
    m_Weight       = 1.0 / (double)m_TotalSamples;
    m_Spacing      = 1.0 / (double)m_NrOf1DSamples;
    m_Delta        = m_Width * m_Spacing;
    // init our filter LUT with the specified parameters
    InitLUT();
  }
  else  {
    m_Weight = m_Spacing = 0.0;
    m_TotalSamples = 0;
  }
}

void CFilter::SetWidth( double XWidth, double YWidth )
{
  if( XWidth != m_Width.x || YWidth != m_Width.y ) {
    m_Width.x = XWidth;
    m_Width.y = YWidth;
    m_Delta        = m_Width * m_Spacing;
    InitLUT();    //  re-create LUT
  }
  else {
    m_Width.x = XWidth;
    m_Width.y = YWidth;
    m_Delta   = m_Width * m_Spacing;
  }
}
