/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	20-Apr-2020
 */

#include "gjob.h"

// ================================ GeometryJob ================================
/** GeometryJob */
GeometryJob::GeometryJob(int nt)
	: geometry(nullptr),
	  running(false)
{
	if (nt>=0) initThreads(nt);
	thread = 0;
} // GeometryJob

/** initialize threadpool
 * @param nt	number of threads, 0=all cpu/cores, 1=synchronous mode
 */
#ifdef THREAD
void GeometryJob::initThreads(int nt)
#else
void GeometryJob::initThreads(int)
#endif
{
#ifdef THREAD
	if (nt==0)	// 0 = use all available threads
		threadpool.init(ThreadPool::num_cores());
	else
	if (nt==1)	// 1 = run in sync mode, no threadpool
		threadpool.init(0);
	else	// n = up to max number of threads to use
		// limit to maximum number of cores
		threadpool.init(Min(nt, ThreadPool::num_cores()));
#endif
} // initThreads

/** stop current job */
void GeometryJob::stop()
{
#ifdef THREAD
	threadpool.stop();
#endif
	if (thread) {
		pthread_join(thread, NULL);
		thread = 0;
	}
	//if (thread.joinable()) thread.join();
	running = false;
} // stop
