/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	20-Apr-2020
 *
 * Standalone parallel job
 */

#ifndef __GEOMETRY_GJOB_H
#define __GEOMETRY_GJOB_H

//#include <thread>
#include <pthread.h>

#include "tool/threadpool.h"

#include "geometry.h"

class Geometry;

// =============================== GeometryJob =================================
class GeometryJob {
public:
const	Geometry*	geometry;	/** geometry class		*/

protected:
#ifdef THREAD
	ThreadPool	threadpool;	/** pool of threads for drawing	*/
#endif
	bool		running;	/** flag if we are running	*/
	//std::thread	thread;		/** job thread idl		*/
	pthread_t	thread;		/** job thread idl		*/

public:
	GeometryJob(int nt=0);
virtual	~GeometryJob() {}

	void	initThreads(int nt);
	int	nthreads() const	{
#ifdef THREAD
			return threadpool.nthreads();
#else
			return 1;
#endif
		}
	bool	isRunning() const	{ return running; }

	/** run method to superclass */
	//void	run() = 0;

	/** stop current job */
	void	stop();

	/** spawn a job in a background thread */
//virtual	void	spawn();

protected:
	/** to be called before the start of the run */
	void	startRun() {
			running = true;
			geometry->lockRead();
		}

	/** end the end of run */
	void	endRun() {
			geometry->unlockRead();
			running = false;
		}
}; // class GeometryJob

#endif
