/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	20-Oct-2019
 */

#ifndef __MATH_SPHERE_H
#define __MATH_SPHERE_H

#include "os.h"
#include "eps.h"
#include "surface.h"

// ================================== Sphere ===================================
/** Sphere surface at location 'center'
 */
class Sphere : public Surface {
public:
	Point	center;		/** center of sphere	*/
	double	R;		/** radius of sphere	*/

public:
	Sphere() : R(0) {}
	Sphere(const Point& p, const double r=0)
		: center(p),
		  R(r)
		 { }

virtual	int	degree() const				{ return 2; }

	/** evaluate gradient at point p */
virtual	Vector	grad(const Point& p) const;

	/** @return closest distance */
virtual	double	distance(const Point& p) const;

	/** @return intersection distance with line p+v*t */
virtual	int	distance(const Point& p, const Vector v, double t[]) const;

virtual	void	translate(const Vector& v)		{ center -= v; }
}; // class Sphere

#endif
