/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_DECORATION_LAYER_H
#define __VIEWER_DECORATION_LAYER_H

#include "layer.h"
#include "graph/bfont.h"
#include "geometry/geo.h"

class Painter;
class Geometry;
class GeometryViewer;

/**
 * DecorationLayer
 *
 * This class draws all the decorations of a viewport and provides an interface
 * to show messages and the progress bar.
 *
 *  The draw method draws the following:
 *  - Grid
 *  - Axes
 *  - Title
 */
class DecorationLayer : public Layer {
public:
	/* flags */
	bool	showAxes;			/** draw axes			*/
	bool	showTitle;			/** flag ON/OFF for title	*/
	bool	showGrid;			/** flag ON/OFF for grid	*/

	/* grid */
	double	grid_x,  grid_y,  grid_z;	/** real positions of grid	*/
	double	grid_dx, grid_dy, grid_dz;	/** real positions of grid	*/
	double	grid_u,  grid_v;		/** starting location of grid	*/
	double	grid_du, grid_dv;		/** step of grid		*/
	char	gridU;				/** axis to plot		*/
	char	gridV;				/** axis to plot		*/
	int	gridLevel;			/** grid level			*/
	BFont	gridFont;			/** drawing font for grid/axes	*/

public:
	DecorationLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v) : Layer(g, k, v) {
			showTitle = true;
			showAxes  = true;
			showGrid  = true;

			grid_du = grid_dv = 100.0;
			gridU = 'U';
			gridV = 'V';
			gridLevel = 200;
			gridFont.load(DEFAULT_FONT);
		}

	void draw(Painter& painter, const dword mask);
	void drawMessage(Painter& painter, const char *msg, dword color, int line);
	void drawProgress(Painter& painter, int percent, const char *msg);

private:
	void drawAxes(Painter& painter);
	void drawGrid(Painter& painter);
}; // DecorationLayer

#endif
