/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_USRBIN_LAYER_H
#define __VIEWER_USRBIN_LAYER_H

#include "layer.h"
#include "geometry/ray.h"
#include "geometry/geo.h"
#include "data/meshdata.h"
#include "geometry/light.h"

class Color;
class Painter;

#define MAXMESH 20

/* =============================== UsrbinLayer ============================== */
class UsrbinLayer : public Layer {
public:
//static	const int	MAXMESH = 20;
protected:
	int		ndata;			/** number of data	*/
	MeshData*	meshdata[MAXMESH];	/** mesh data objects	*/
	int		_alpha[MAXMESH];	/** transparency	*/
	int		_palette[MAXMESH];	/** palette index	*/
	bool		_showErr[MAXMESH];	/** display error instead of data */

public:
	UsrbinLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);
	~UsrbinLayer();

	/* set/get */
	int	alpha(int i)		const	{ return _alpha[i]; }
	void	alpha(int i, int a)		{ _alpha[i] = a; }

	int	palette(int i)		const	{ return _palette[i]; }
	void	palette(int i, int p)		{ _palette[i] = p; }

	bool	showErrors(int i) const		{ return _showErr[i]; }
	void	showErrors(int i, bool bFlag)	{ _showErr[i] = bFlag; }

	bool	load(int idx, const char* filename, const int det);
	bool	checker(int idx, const bool rphiz,
			double axlow, double axhigh, const int anx,
			double aylow, double ayhigh, const int any,
			double azlow, double azhigh, const int anz);

	MeshData* operator [](const int i)	{ return meshdata[i]; }

	void	draw(Painter& painter);
	byte	shade(const Point& hit, Color& color) const;
	byte	bumpShade(Ray& hitRay, const VLight& VL, double scaling, int direction, Color& color) const;
	void	regionColorFromMeshData(int idx);
}; // UsrbinLayer

#endif
